import glob

from pathlib import Path

import toml, rich


_EXCLUSIONS = ["__init__"]

tom = toml.load(Path(__file__).parent / "pyproject.toml")
version = tom["tool"]["poetry"]["version"]
package_name = tom["tool"]["poetry"]["name"]

rich.print(
    f"[red]Rob's Utility Scripts[/red]\n[green]Version: {version}[/green]\n\nAvailable scripts (pass -h for CLI usage):"
)
print(
    "\n".join(
        [
            Path(elem).stem
            for elem in glob.glob(f"{Path(__file__).parent}/*.py")
            if Path(elem).stem not in _EXCLUSIONS
        ]
    )
)



"""(C) Rob Olson"""
# pylint: disable = C0330
# pylint: disable = multiple-imports
import os, shelve, sys, datetime, argparse, subprocess, appdirs

from rich.progress import track

from pathlib import Path

import praw

_THIS_FILE = Path(sys.argv[0])
_DB_FILE = Path(appdirs.user_data_dir()) / "robolson" / "reddit_archive" / "comments.db"

if not _DB_FILE.exists():
    os.makedirs(_DB_FILE.parent, exist_ok=True)

# Create the parser
_ARGPARSER = argparse.ArgumentParser(
    prog=sys.argv[0],
    allow_abbrev=True,
    add_help=True,
    description="Archive reddit comments.",
    epilog="(C) Rob",
)

_ARGPARSER.add_argument(
    "-t", "--text", action="store_true", help="generate text file ONLY"
)
_ARGPARSER.add_argument(
    "-o", "--overwrite", action="store_true", help="overwrite existing database entries"
)
_ARGPARSER.add_argument(
    "-u",
    "--user",
    metavar="user",
    nargs=1,
    action="store",
    type=str,
    help="specify user",
)

_ARGPARSER.add_argument(
    "-p",
    # "--pwd",
    "--password",
    metavar="pwd",
    nargs=1,
    action="store",
    type=str,
    help="specify password",
)

_ARGPARSER.add_argument(
    "-f",
    "--full",
    action="store_true",
    help="Include 'top' and 'controversial' posts in praw request.",
)

_ARGPARSER.add_argument(
    "-c",
    "--config",
    action="store_true",
    help="Manually configure reddit credentials for this user agent.",
)

_ARGS = _ARGPARSER.parse_args()

if _ARGS.config:
    _REDDIT_ID = input("\nEnter a reddit account ID.\n> ")
    _REDDIT_SECRET = input(
        "\nEnter the reddit secret for this account\n(located at https://www.reddit.com/perfs/apps)\n> "
    )
    choice = input("Save credentials? Y/N\n>")
    if choice in ["yes", "y", "Y"]:
        subprocess.run(["setx", "REDDIT_ID", _REDDIT_ID])
        subprocess.run(["setx", "REDDIT_SECRET", _REDDIT_SECRET])

else:
    # located at https://www.reddit.com/prefs/apps
    _REDDIT_ID = os.environ["REDDIT_ID"]
    _REDDIT_SECRET = os.environ["REDDIT_SECRET"]

match (_ARGS.user, _ARGS.password):
    case (None, None) | (_, None) | (None, _):
        try:
            _REDDIT_USERNAME = os.environ["REDDIT_USERNAME"]
            _REDDIT_PASSWORD = os.environ["REDDIT_PASSWORD"]
        except KeyError:
            _REDDIT_USERNAME = input("Enter reddit username: ")
            _REDDIT_PASSWORD = input("Enter reddit password: ")
            choice = input("\nSave login info (WARNING: NOT SECURE)? Y/N\n> ")
            if choice in ["yes", "YES", "y", "Y"]:
                subprocess.run(["setx", "REDDIT_USERNAME", _REDDIT_USERNAME])
                subprocess.run(["setx", "REDDIT_PASSWORD", _REDDIT_PASSWORD])
    case (u, p) if u and p:
        _REDDIT_USERNAME = u
        _REDDIT_PASSWORD = p
    case _:
        print("DUH")


_REDDIT = praw.Reddit(
    client_id=_REDDIT_ID,
    client_secret=_REDDIT_SECRET,
    user_agent="long_comment_aggregator",
    username=_REDDIT_USERNAME,
    password=_REDDIT_PASSWORD,
)


def generate_text():
    now = datetime.datetime.now()
    with shelve.open(str(_DB_FILE)) as db, open(
        f"reddit_archive_{now.year}_{now.month}_{now.day}.txt",
        "w",
        encoding="utf-8",
    ) as fp:
        sorted_comments = reversed(
            sorted(
                [v for k, v in db.items()],
                key=lambda x: x["created_utc"],
            )
        )

        for comment in sorted_comments:
            try:
                parent_author = comment["parent_author"]
            except KeyError:
                parent_author = "?"
            fp.write(
                f"""
======
http://reddit.com{comment['permalink']}
{comment['human_time']} ({comment['ups']})
======
{parent_author}: {comment['parent_body']}
======
{comment['body']}
======\n\n"""
            )


def main():  # pylint: disable=missing-function-docstring
    if _ARGS.text:
        generate_text()
        exit(0)

    try:
        me = _REDDIT.user.me()  # pylint: disable=invalid-name
    except BaseException:
        input("Invalid login credentials.")
        exit(1)

    new = me.comments.new(limit=None)
    top = me.comments.top(limit=None)
    contro = me.comments.controversial(limit=None)

    with shelve.open(str(_DB_FILE)) as db:  # pylint: disable=invalid-name
        prev = db.keys()
        print("Archiving 'new'...")
        count = 0
        for comment in new:
            print(count, end="\r", flush=True)
            count += 1
            if _ARGS.overwrite:
                try:
                    del db[comment.id]
                except KeyError:
                    pass

            if comment.id not in prev and len(comment.body) > 100:
                parent = comment.parent()
                db[comment.id] = {
                    "id": comment.id,
                    "body": comment.body,
                    "ups": comment.ups,
                    "downs": comment.downs,
                    "permalink": comment.permalink,
                    "parent_author": str(getattr(parent, "author", None)),
                    "parent_body": getattr(parent, "body", None),
                    "created_utc": comment.created_utc,
                    "human_time": datetime.datetime.fromtimestamp(
                        comment.created_utc
                    ).isoformat(),
                }

        if not _ARGS.full:
            generate_text()
            exit(0)

        print("Archiving 'top'...")
        count = 0
        for comment in top:
            print(count, end="\r", flush=True)
            count += 1
            if _ARGS.overwrite:
                try:
                    del db[comment.id]
                except KeyError:
                    pass

            if comment.id not in prev and len(comment.body) > 100:
                parent = comment.parent()

                db[comment.id] = {
                    "id": comment.id,
                    "body": comment.body,
                    "ups": comment.ups,
                    "downs": comment.downs,
                    "permalink": comment.permalink,
                    "parent_author": str(getattr(parent, "author", None)),
                    "parent_body": getattr(parent, "body", None),
                    "created_utc": comment.created_utc,
                    "human_time": datetime.datetime.fromtimestamp(
                        comment.created_utc
                    ).isoformat(),
                }

        print("Archiving 'controversial'...")
        for comment in contro:
            if _ARGS.overwrite:
                try:
                    del db[comment.id]
                except KeyError:
                    pass

            if comment.id not in prev and len(comment.body) > 100:
                parent = comment.parent()

                db[comment.id] = {
                    "id": comment.id,
                    "body": comment.body,
                    "ups": comment.ups,
                    "downs": comment.downs,
                    "permalink": comment.permalink,
                    "parent_author": str(getattr(parent, "author", None)),
                    "parent_body": getattr(parent, "body", None),
                    "created_utc": comment.created_utc,
                    "human_time": datetime.datetime.fromtimestamp(
                        comment.created_utc
                    ).isoformat(),
                }

    generate_text()


if __name__ == "__main__":
    main()



import glob
import pathlib
import os
import time
import asyncio
import argparse
import logging
import sys
import subprocess
import multiprocessing
import time
import appdirs
import random

import pydub
import rich

from rich.progress import track
from pydub.utils import mediainfo

# Create the parser
my_parser = argparse.ArgumentParser(
    prog=sys.argv[0],
    allow_abbrev=True,
    add_help=True,
    description="Concatenates audio files to m4b using ffmpeg.",
    epilog="(C) Rob",
)

# Add the arguments
my_parser.add_argument(
    "-f",
    "--filetype",
    metavar="filetype",
    nargs=1,
    default=".mp3",
    choices=[".mp3", ".wav", ".ogg"],
    action="store",
    type=str,
    help="the filetype to concatenate, e.g., '.mp3'",
)

my_parser.add_argument(
    "-p",
    "--path",
    metavar="path",
    nargs=1,
    default=".",
    action="store",
    type=str,
    help="the path to files to be concatted",
)

my_parser.add_argument(
    "-c",
    "--cpus",
    metavar="cpus",
    default=1,
    action="store",
    type=int,
    help="the number of processor cores to utilize",
)

my_parser.add_argument(
    "-n",
    "--number",
    metavar="number",
    default=0,
    action="store",
    type=int,
    help="the number of folders to limit execution on",
)

my_parser.add_argument(
    "-s",
    "--start",
    metavar="start",
    default=0,
    action="store",
    type=int,
    help="start execution on nth folder (ignoring previous)",
)

my_parser.add_argument('--safe',
                       action='store_false',
                       help='force ffmpeg to overwrite existing files')

my_parser.add_argument('--command',
                       action='store_true',
                       help='Only print the corresponding ffmpeg command(s)')

my_parser.add_argument('-i',
                       '--interactive',
                       action='store_true',
                       help='supply arguments manually via prompt')

my_parser.add_argument('-l',
                       '--local',
                       action='store_true',
                       help='store temporary files on local machine')

# Execute the parse_args() method
_ARGS = my_parser.parse_args()

_PROMPT = f"rob.{pathlib.Path(__file__).stem}> "

_PATH = pathlib.Path(_ARGS.path)
_FILETYPE = _ARGS.filetype
_SAFE = _ARGS.safe
_CPUS = _ARGS.cpus
_COMMAND = _ARGS.command
_TEMP_FOLDER = pathlib.Path(f"{appdirs.user_data_dir()}") / "robolson" / "ffmpeg" / "temp"
_COMMAND_FILE = pathlib.Path(os.getcwd()) / "ffmpeg_commands.ps1"

def command_only(folder):
    os.chdir(folder)

    mp3s = glob.glob(f"*{_FILETYPE}")
    mp3s = [pathlib.Path(elem) for elem in mp3s if elem[0] != '~']

    if not mp3s:
        return

    concated = "|".join([str(elem.absolute()) for elem in mp3s])
    command = [
        "ffmpeg",
        "-i",
        f"'concat:{concated}'",
        "-movflags", # Carry metadata over
        "use_metadata_tags",
        "-c:a", # audio cocec
        "aac",
        "-b:a", # bitrate
        "64k",
        "-c:v",
        "copy",
        f"{pathlib.Path(mp3s[0]).stem}.m4b",
        "-y",
    ]


    with open(_COMMAND_FILE, "a") as fp:
        fp.write(" ".join(command)+"\n")

    rich.print("[green]"+" ".join(command))

    return

def spin_up(folder):
    """Spin up an ffmpeg process in target folder.

    Args:
        folder: path to folder that contains audio files
    """

    os.chdir(folder)

    global _SAFE

    mp3s = glob.glob(f"*{_FILETYPE}")
    mp3s = [pathlib.Path(elem) for elem in mp3s if elem[0] != '~']


    if not mp3s:
        return

    concated = "|".join([str(elem.absolute()) for elem in mp3s])

    salt = ""
    if _ARGS.local:
        salt = "".join(random.choices("abcdefghijklmnopqrstuvwxyz", k=10))
        os.makedirs(_TEMP_FOLDER, exist_ok=True)
        _temp_file = _TEMP_FOLDER / f"{salt}~{pathlib.Path(mp3s[0]).stem}-full{_FILETYPE}"
    else:
        _temp_file = f"~{pathlib.Path(mp3s[0]).stem}-full{_FILETYPE}"


    command = [
        "ffmpeg",
        "-i",
        str(_temp_file), # f"~{pathlib.Path(mp3s[0]).stem}-full{_FILETYPE}",
        "-movflags", # Carry metadata over
        "use_metadata_tags",
        "-c:a", # audio cocec
        "aac",
        "-b:a", # bitrate
        "64k",
        "-c:v",
        "copy",
        f"{pathlib.Path(mp3s[0]).stem}.m4b",
        "-y"
    ]

    if _SAFE:
        command.remove('-y')

    bit_rate = mediainfo(mp3s[0])['bit_rate']

    # Merge audio files

    if len(mp3s)==1:
        combined = pydub.AudioSegment.from_mp3(mp3s[0])

    else:
        combined = pydub.AudioSegment.empty()
        match _FILETYPE:
            case ".mp3":
                for file in track(mp3s):
                    combined += pydub.AudioSegment.from_mp3(file)
                    # combined += pydub.AudioSegment.from_file(file, "mp3")

            case ".wav":
                for file in track(mp3s):
                    combined += pydub.AudioSegment.from_wav(file)

            case ".ogg":
                for file in track(mp3s):
                    combined += pydub.AudioSegment.from_ogg(file)

    rich.print(f"[yellow]Writing temp file ({pathlib.Path(_temp_file).absolute()}) to disk.")
    # combined.export(_temp_file, tags={"test":"TEST"}, bitrate=bit_rate)
    combined.export(
    _temp_file,
    format="ipod",
    # bitrate=bit_rate,
    parameters=["-movflags", "use_metadata_tags", "-c:a", "aac", "-c:v", "copy", "-b:a", "64k"],
)

    # End of Merge audio files

    count = 0
    while not pathlib.Path(_temp_file).exists():
        count += 1
        rich.print(f"[red]Merged file not found.[/red]  Retry {count}/5 in 10s.", end="\r", flush=True)
        if count > 6:
            print("Merged file not found.  Exiting...")
            exit(1)
        time.sleep(10)

    rich.print(f"\n[green]Merged file ({pathlib.Path(_temp_file).absolute()}) created.[/green]\n")

    # Convert audio files

    rich.print("\nRunning the following command:\n[yellow]" + " ".join(command) + "[/yellow]\n")
    subprocess.run(command, shell=True)
    # End of Convert audio files

    if salt:
        rich.print(f"[red]Removing temp file ({pathlib.Path(_temp_file).absolute()}).")
        os.remove(_temp_file)

    rich.print(f"[green]Finished {folder}.\n")

def interact():
    global _PATH
    global _FILETYPE
    global _SAFE
    global _CPUS
    global _COMMAND

    choice = input(f"Enter path to work on [default: {_PATH.absolute()}].\n{_PROMPT}")
    if choice:
        _PATH = pathlib.Path(choice)

    rich.print(f"[yellow]Working on '{_PATH.absolute()}'.\n")


    choice = input(f"Automatically overwrite existing files [default: {_SAFE}]? y/n\n{_PROMPT}")
    if choice in ['y', "Y", 'yes', "YES"]:
        _SAFE = False

    elif not choice:
        pass
    else:
        _SAFE = True

    if not _SAFE:
        rich.print("[red]Overwriting...\n")
    else:
        rich.print("[green]NOT overwriting...\n")

    choice = input(f"Which file type to work on [default: {_FILETYPE}]?\n 1.) *.mp3\n 2.) *.wav\n 3.) *.ogg\n{_PROMPT}")
    match choice:
        case "1" | "*.mp3" | ".mp3" | "mp3":
            _FILETYPE = ".mp3"

        case "2" | "*.wav" | ".wav" | "wav":
            _FILETYPE = ".wav"

        case "3" | "*.ogg" | ".ogg" | "ogg":
            _FILETYPE = ".ogg"

        case _:
            pass

    rich.print(f"[yellow]Working on {_FILETYPE}'s.\n")

    folders = set()
    os.chdir(_PATH)
    for file in glob.glob(f"**/*{_FILETYPE}", recursive=True):
        if _SAFE:
            try:
                os.remove(pathlib.Path(f"{pathlib.Path(file).parent}/{pathlib.Path(file).stem}.m4b"))
            except FileNotFoundError as e:
                pass
        target = pathlib.Path(file).absolute()
        folders.add(target.parent)

    all_folders = sorted(list(folders))
    folders = all_folders[:]

    final_choice = True
    while final_choice:
        rich.print(f"Found folders with {_FILETYPE}'s:")
        for count, folder in enumerate(all_folders):
            if folder in folders:
                rich.print(f"[green]{count+1}.) {folder}")
            else:
                rich.print(f"[red]{count+1}.) {folder}")

        choice = input(f"Toggle execution of folders by number or press Enter to continue.\n{_PROMPT}")
        try:
            choice = int(choice)
            if choice <= len(all_folders):
                if all_folders[choice-1] in folders:
                    folders.remove(all_folders[choice-1])
                else:
                    folders.insert(choice-1, all_folders[choice-1])
            else:
                rich.print(f"Found folders with {_FILETYPE}'s:")
                for count, folder in enumerate(all_folders):
                    if folder in folders:
                        rich.print(f"[green]{count+1}.) {folder}")
                    else:
                        rich.print(f"[red]{count+1}.) {folder}")
                final_choice = input(f"Continue? (y/n)\n{_PROMPT}")
                if final_choice not in ["n", "no", "N", "No", "NO"]:
                    final_choice = False

        except (SyntaxError, ValueError):
            final_choice = input(f"Continue? (y/n)\n{_PROMPT}")
            if final_choice not in ["n", "no", "N", "No", "NO"]:
                final_choice = False

    choice = input(f"How many CPU cores to use [default: {_CPUS}]?\n{_PROMPT}")
    while True:
        try:
            if not choice:
                break
            else:
                _CPUS = int(choice)
                break
        except (TypeError, ValueError):
            rich.print(f"[red]Must supply an integer.[/red]\nHow many CPU cores to use \[default: {_CPUS}]?\n")
            choice = input(f"{_PROMPT}")

    rich.print(f"[yellow]Using {_CPUS} core{'s' if _CPUS>1 else ''}.\n")

    choice = input(f"(1) Execute commands or (2) Generate command file [default {'1' if _COMMAND else '1'}]?\n{_PROMPT}")
    match choice:
        case '1':
            _COMMAND = False
        case '2':
            _COMMAND = True
            open(_COMMAND_FILE, "w").close()
        case _:
            pass

    return folders

def main():
    folders = set()

    global _SAFE

    if _COMMAND:
        open(_COMMAND_FILE, "w").close()

    if _ARGS.interactive:
        folders = interact()

    else:
        os.chdir(_PATH)
        for file in glob.glob(f"**/*{_FILETYPE}", recursive=True):
            if _SAFE:
                try:
                    os.remove(pathlib.Path(f"{pathlib.Path(file).parent}/{pathlib.Path(file).stem}.m4b"))
                except FileNotFoundError as e:
                    pass
            target = pathlib.Path(file).absolute()
            folders.add(target.parent)

        folders = sorted(list(folders))

        # optionally slice the list of folders
        match (bool(_ARGS.number), bool(_ARGS.start)):
            case (True, False):
                folders = folders[0:_ARGS.number]
            case (False, True):
                folders = folders[_ARGS.start-1:]
            case (True, True):
                folders = folders[_ARGS.start-1:_ARGS.start-1+_ARGS.number]
            case _:
                pass

    input(
        f"{'Compiling' if _COMMAND else 'Executing'} on \n * "
        + "\n * ".join([str(folder) for folder in folders])
        + f". \n Using {_CPUS} core. Ok?\n{_PROMPT}"
    )

    if _COMMAND:
        with multiprocessing.Pool(_CPUS) as pool:
            pool.map_async(command_only, folders).get()
    else:
        with multiprocessing.Pool(_CPUS) as pool:
            pool.map_async(spin_up, folders).get()

if __name__ == "__main__":
    main()



"""Organizes files in a directory by their extension."""

# Robert Olson
# pylint: disable=line-too-long

import os
import glob
import datetime
import shutil
import sys
import re
import argparse
import json
import appdirs
import shutil

import rich.traceback
import rich
import toml

from pathlib import Path

rich.traceback.install(show_locals=True)

DEBUG = False
HANDLE_MISC = True

THIS_FILE = Path(__file__)

CONFIG_FILE = Path(appdirs.user_config_dir()) / "robolson" / "config" / "clean.toml"

if CONFIG_FILE.exists():
    with (open(CONFIG_FILE, "r")) as fp:
        SETTINGS = toml.load(fp)

else:
    base_config = THIS_FILE.parent / "config" / "clean.toml"
    os.makedirs(CONFIG_FILE.parent, exist_ok=True)
    shutil.copyfile(base_config, CONFIG_FILE)
    with (open(CONFIG_FILE, "r")) as fp:
        SETTINGS = toml.load(fp)

# Number of files in a folder that prompts more sorting
CROWDED_FOLDER = SETTINGS["CROWDED_FOLDER"]


FILE_TYPES = SETTINGS["FILE_TYPES"]

EXCLUSIONS = SETTINGS["EXCLUSIONS"]

MONTHS = SETTINGS["MONTHS"]
MONTHS.insert(0, None)

PROMPT = f"rob.{THIS_FILE.stem}> "


def handle_files(files: list, folder: str = "misc", month: bool = False):
    """Organizes files by last modified date."""
    choice = ""

    for file in files:
        last_modified = datetime.datetime.fromtimestamp(os.path.getmtime(file))
        f_day = last_modified.day
        f_month = MONTHS[last_modified.month]

        file_size = os.stat(file).st_size

        f_year = last_modified.year
        if file_size > 150_000_000:
            target_folder = os.path.join("Large_Files", str(f_year))
        else:
            target_folder = os.path.join(folder, f"{folder} {str(f_year)}")

        if month:
            target_folder = os.path.join(
                target_folder, f"{folder} {last_modified.month} ({f_month}) {f_year}"
            )
        os.makedirs(target_folder, exist_ok=True)
        # try:
        #     os.makedirs(target_folder, exist_ok=True)
        # except:
        #     pass

        while choice not in ["y", "yes", "n", "no", "a", "all", "d", "del"]:
            choice = input(
                f"mv '{file}' '{target_folder}\\{os.path.split(file)[1]}'\n(y)es/(n)o/yes_to_(a)ll/(d)el?\n{PROMPT}"
            )

        if choice in ["y", "yes"]:
            try:
                shutil.move(file, target_folder)

            # File of Same Name Has Already Been Moved To Folder
            except shutil.Error:
                print(
                    f"Renamed '{file}' to '{f_month} {f_day} ({datetime.datetime.now().time().microsecond}) COPY {file}'.\n"
                )
                # os.rename(file, target_folder + "\\COPY " + file)
                Path(file).rename(
                    target_folder
                    + f"\\{Path(file).stem} {MONTHS[datetime.datetime.now().month]} {datetime.datetime.now().day} ({int((datetime.datetime.now() - datetime.datetime.min).total_seconds())}) COPY{Path(file).suffix}"
                )
                choice = ""

        elif choice in ["a", "all"]:
            shutil.move(file, target_folder)

        elif choice in ["n", "no"]:
            choice = ""

        elif choice in ["d", "del"]:
            os.makedirs("delete_me", exist_ok=True)
            shutil.move(file, os.path.normpath(f"delete_me/{file}"))
            # os.remove(file)
            choice = ""


def remove_empty_dir(path: str | Path):
    """Remove empty folder."""

    try:
        os.rmdir(path)
        print(f"Removing empty folder ({path}).")
    except OSError as e:
        if DEBUG:
            print(f"Could not remove folder: {e}")
        else:
            pass


def remove_empty_dirs(path: str | Path):
    """Recursively remove empty folders."""

    for trunk, dirnames, filenames in os.walk(path, topdown=False):
        for dirname in dirnames:
            remove_empty_dir(os.path.realpath(os.path.join(trunk, dirname)))

    remove_empty_dir(path)


# MAIN()
def main():

    # Create the parser
    my_parser = argparse.ArgumentParser(description="Clean up a folder.")

    # Add the arguments
    my_parser.add_argument(
        "Path",
        metavar="path",
        nargs="?",
        default=".",
        action="store",
        type=str,
        help="the path to list",
    )

    args = my_parser.parse_args()

    CLI_path = args.Path

    if not os.path.isdir(CLI_path):
        rich.print(f"[red on black]The specified path ({CLI_path}) does not exist.")
        exit(1)
        root = input(
            f"Clean current directory ({os.getcwd()})?\nPress Enter to continue or enter a new path to clean.\n{PROMPT}"
        )

    else:
        root = CLI_path

    root = os.path.normpath(root)

    os.chdir(root)

    all_files = glob.glob("*.*")

    for file_name in all_files:
        if file_name in EXCLUSIONS:
            all_files.remove(file_name)

    file_groups = {}

    ARCHIVE_FOLDERS = list(FILE_TYPES.keys())

    # put all files with same extension group into one list
    # and put that list in the file_groups dictionary
    # FOR EXAMPLE
    # file_groups["media"] will contain a list of all pictures in CWD
    # file_groups["zip files"] contain a list of all compressed archives in CWD
    # etc

    for file_type, extension_list in FILE_TYPES.items():
        if not extension_list:
            continue

        extension_pattern = re.compile(
            "(" + "|".join(extension_list) + ")$", re.IGNORECASE
        )
        file_groups[file_type] = [
            file_name
            for file_name in all_files
            if re.search(extension_pattern, file_name)
        ]

        for file in file_groups[file_type]:
            all_files.remove(file)

    # Any file-types not explicitly handled are moved to the miscellaneous folder
    if HANDLE_MISC and all_files:
        try:
            file_groups["misc"].extend(all_files)
        except KeyError:
            file_groups["misc"] = all_files
        print(f"moved {all_files}")

    # Do not target THIS file
    if __file__ in file_groups["programming"]:
        file_groups["programming"].remove(__file__)

    # Do not target THIS file
    if THIS_FILE in file_groups["programming"]:
        file_groups["programming"].remove(THIS_FILE)

    file_count = sum([len(file_group) for file_type, file_group in file_groups.items()])

    print(f"({file_count}) files/folders to move.\n")

    # Handles all files in file_groups
    for file_type, file_group in file_groups.items():
        if not file_group:
            continue

        handle_files(file_group, file_type)

        # Each file-type-folder should have one or more year folders (e.g., 'media/2020')
        year_folders = glob.glob(file_type + "/* ????")

        # Check year folders for crowdedness
        for year in year_folders:
            sorted_files = glob.glob(year + "/*.*")
            pre_sorted_files = glob.glob(year + "/*/*.*")

            if sorted_files and (
                len(sorted_files) + len(pre_sorted_files) > CROWDED_FOLDER
            ):
                choice = input(
                    f"{year} has {len(sorted_files)} top-level files and {len(pre_sorted_files)} already sorted files.  Sort by month (y/n)?\n{PROMPT}"
                )
                if choice in ["y", "yes"]:
                    handle_files(sorted_files, file_type, month=True)

    # Check for extra folders not generated by this program
    extra_folders = [
        elem
        for elem in glob.glob("*")
        if not Path(elem).suffix and elem not in ARCHIVE_FOLDERS
    ]

    move_folders = False
    if extra_folders:
        choice = input(
            "\nNon-archive folders detected.\n* {}\nArchive them (y/n)?\n{}".format(
                "\n* ".join(extra_folders), PROMPT
            )
        )
        if choice in ["y", "yes"]:
            move_folders = True

    if move_folders:
        for extra_folder in extra_folders:
            choice = input(f"{Path(extra_folder).resolve()}\nMove (y/n)?\n{PROMPT}")

            if choice in ["y", "yes"]:
                for i, default_folder in enumerate(ARCHIVE_FOLDERS):
                    print(f"\n{i+1}.) {default_folder}")
                target_folder = input(
                    f"\nmv '{extra_folder}' ???\nDestination?\n{PROMPT}"
                )
                try:
                    if int(target_folder) in list(range(1, len(ARCHIVE_FOLDERS) + 1)):
                        target_folder = ARCHIVE_FOLDERS[int(target_folder) - 1]
                except ValueError:
                    pass

                handle_files([extra_folder], folder=target_folder)
                choice = ""

    # Removes empty folders, except in programming because of git clone
    for target_folder in ARCHIVE_FOLDERS:
        remove_empty_dirs(os.path.join(root, target_folder))
        # if target_folder != "programming":
        #     remove_empty_dirs(os.path.join(root, target_folder))
        # else:
        #     remove_empty_dir(target_folder)


# END OF MAIN()

if __name__ == "__main__":
    main()
